package controllers

import (
	"errors"
	"net/http"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
	"time"
)

// Authenticate 权限验证
func (c *BaseController) authenticate() {
	serv := service.NewSysServ(c.Context)

	// 鉴权 - 并初始化上下文
	res := serv.AuthAndInitCtx(c.Ctx)

	// 设置 TOKEN
	c.setToken()

	if res != nil {
		code := http.StatusOK
		if res["code"] != nil {
			code = res["code"].(int)
		}

		if code != http.StatusOK {
			err := res["error"].(error)
			data := map[string]interface{}{}

			if res["message"] != nil {
				data = res["message"].(map[string]interface{})
			}

			c.ResponseData(data, err, code)
		}
	}
}

// setToken 设置 TOKEN
// 15 分钟后过期
func (c *BaseController) setToken() {
	var token *utils.JWTToken
	exp := time.Now().Local().Add(15 * time.Second)

	if c.Context.Get("user") != nil {
		user := c.Context.Get("user").(model.SysUser)

		token = &utils.JWTToken{
			Guest:  false,
			ID:     user.UserId,
			Expire: exp,
		}
	} else if c.Context.Get("userMap") != nil {
		userMap := c.Context.Get("userMap").(model.TaUserMapping)

		token = &utils.JWTToken{
			Guest:  false,
			ID:     userMap.Openid,
			Expire: exp,
		}
	} else {
		token = &utils.JWTToken{
			Guest:  true,
			Expire: exp,
		}
	}

	tokenEncode, err := utils.CreateToken(token.ToMap())
	if err != nil {
		utils.LogError("系统生成 Token 失败: " + err.Error())
		c.ResponseError(
			errors.New("系统异常, 请退出重试"),
			http.StatusInternalServerError,
		)
	}

	c.Ctx.Output.Header(utils.TokenHeader, utils.TokenSchema+" "+tokenEncode)
}
